using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.Serialization;

namespace Set
{
	[Serializable]
    public class HashSet<T> : Set<T>
    {

        private Dictionary<T, bool> set;

        public HashSet()
        {
            set = new Dictionary<T, bool>();
        }

        public HashSet(HashSet<T> other)
            : this()
        {
            foreach (T key in other)
                Add(key);
        }

        public override object Clone()
        {
            return new HashSet<T>(this);
        }

        public override bool Contains(T item)
        {
            return set.ContainsKey(item);
        }

        public override void Add(T item)
        {
            if (!set.ContainsKey(item)) set.Add(item, true);
            else set[item] = true;
        }

        public override void Remove(T item)
        {
            if (set.ContainsKey(item)) set.Remove(item);
        }

        public override void Union(Set<T> b)
        {
            foreach (T item in b)
            {
                Add(item);
            }
        }

        public override void Intersection(Set<T> b)
        {
            List<T> falsify = new List<T>();
            foreach (T key in this)
            {
                if (!b.Contains(key)) falsify.Add(key);
            }
            foreach (T key in falsify)
            {
                Remove(key);
            }
        }

        public override IEnumerator<T> GetEnumerator()
        {
            foreach (T key in set.Keys)
            {
                yield return key;
            }
        }

		public override int Count
		{
			get
			{
				return set.Count;
			}
		}

        public override List<T> AsList()
        {
            return new List<T>(set.Keys);
        }
    }
}
